#include <stdio.h>
#include <stdlib.h>
#include <gtk/gtk.h>

/* GtkWidget is the storage type for widgets */
/* This widgets are global */
GtkWidget *window, *mainbox, *upperbox, *lowerbox, *leftbox , *rightbox ,*size ,*length,
          *bitrate, *output, *sizelabel ,*lengthlabel, *bitratelabel,
          *outputlabel, *logo ;

void textchange()
{
	int size_num, length_num, bitrate_num;
	long output_num;
	char out_char[256];

	size_num = atoi (gtk_entry_get_text(GTK_ENTRY(size)));
	length_num = atoi (gtk_entry_get_text(GTK_ENTRY(length)));
	bitrate_num = atoi (gtk_entry_get_text(GTK_ENTRY(bitrate)));

	/* checking for invalid values */
	if ( size_num < 1 ) {
	    size_num = 1; 
	    gtk_entry_set_text(GTK_ENTRY(size),"1");
	}
	if ( length_num < 1 ) {
	    length_num = 1; 
	    gtk_entry_set_text(GTK_ENTRY(length),"1");
	}
	if ( bitrate_num < 8 ) {
	    bitrate_num = 8; 
	    gtk_entry_set_text(GTK_ENTRY(bitrate),"8");
	}

	/* here we finaly calculate the bitrate */
	output_num = (size_num * 8 * 1024) / (length_num * 60) - bitrate_num;

	sprintf(out_char, "%d" , output_num);

/*	printf (out_char);  */

	gtk_entry_set_text(GTK_ENTRY(output), out_char );
}

int main( int   argc, char *argv[] )
{
    
    /* This is called in all GTK applications. Arguments are parsed
     * from the command line and are returned to the application. */
    gtk_init (&argc, &argv);
    
    /* create a new window */
    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_window_set_resizable (GTK_WINDOW (window), FALSE); 

    /* Here we connect the "destroy" event to a signal handler.  
     * This event occurs when we call gtk_widget_destroy() on the window,
     * or if we return FALSE in the "delete_event" callback. */
    g_signal_connect (G_OBJECT (window), "destroy",
		      G_CALLBACK (exit), NULL); 

    gtk_window_set_title (GTK_WINDOW (window), "bitrate calculator");
    gtk_container_set_border_width (GTK_CONTAINER (window), 0);

    /* window end */
    
    /* creating the container for the widgets and activating it*/
    mainbox = gtk_vbox_new (FALSE, 0);
    gtk_container_add (GTK_CONTAINER (window), mainbox);

    upperbox = gtk_hbox_new (FALSE, 3);
    gtk_container_add (GTK_CONTAINER (mainbox), upperbox);       

    lowerbox = gtk_hbox_new (FALSE, 3);
    gtk_container_add (GTK_CONTAINER (mainbox), lowerbox);       

    leftbox = gtk_vbox_new (FALSE, 3);
    gtk_container_add (GTK_CONTAINER (lowerbox), leftbox);       

    rightbox = gtk_vbox_new (FALSE, 3);
    gtk_container_add (GTK_CONTAINER (lowerbox), rightbox);

    /* container end */

    /* put the logos on top */
    logo = gtk_image_new_from_file ("/usr/local/share/brcalc/logo.png");
    gtk_misc_set_alignment (GTK_MISC (logo), 0.5 , 0);
    gtk_container_add (GTK_CONTAINER (upperbox), logo);

    /* Creates the inputfields */
    size = gtk_entry_new ();
    gtk_entry_set_max_length (GTK_ENTRY (size), 6);
    gtk_entry_set_text(GTK_ENTRY (size),"640");
    gtk_entry_set_width_chars (GTK_ENTRY (size), 6);
    
    length = gtk_entry_new ();
    gtk_entry_set_max_length (GTK_ENTRY (length), 6);
    gtk_entry_set_text(GTK_ENTRY (length),"90");
    gtk_entry_set_width_chars (GTK_ENTRY (length), 6);

    bitrate = gtk_entry_new ();
    gtk_entry_set_max_length (GTK_ENTRY (bitrate), 6);
    gtk_entry_set_text(GTK_ENTRY (bitrate),"128");
    gtk_entry_set_width_chars (GTK_ENTRY (bitrate), 6);
    
    /* Creates the outputfield */
    output = gtk_entry_new ();
    gtk_entry_set_max_length (GTK_ENTRY (output), 6);
    gtk_entry_set_text (GTK_ENTRY (output), "842");
    gtk_entry_set_width_chars (GTK_ENTRY (output), 6);

    /* we don't want the output editable */
    gtk_editable_set_editable (GTK_EDITABLE (output), FALSE);


    /* Creates the labels */
    sizelabel = gtk_label_new ("Size in MB\n");
    gtk_misc_set_alignment (GTK_MISC (sizelabel),0,0);
    lengthlabel = gtk_label_new ("Duration in minutes\n");
    gtk_misc_set_alignment (GTK_MISC (lengthlabel), 0, 0);
    bitratelabel = gtk_label_new ("Audio bitrate in kbps\n");
    gtk_misc_set_alignment (GTK_MISC (bitratelabel), 0, 0);
    outputlabel = gtk_label_new ("DIVX4 bitrate\n");
    gtk_misc_set_alignment (GTK_MISC (outputlabel), 0, 0);

    /* This packs the entrys into the rightbox */
    gtk_container_add (GTK_CONTAINER (rightbox), size);
    gtk_container_add (GTK_CONTAINER (rightbox), length);
    gtk_container_add (GTK_CONTAINER (rightbox), bitrate);
    gtk_container_add (GTK_CONTAINER (rightbox), output);
    
    /* Selects the size entry */
    gtk_editable_select_region (GTK_EDITABLE (size), 0, -1);

    /* This packs the lables into the leftbox */
    gtk_container_add (GTK_CONTAINER (leftbox), sizelabel);
    gtk_container_add (GTK_CONTAINER (leftbox), lengthlabel);
    gtk_container_add (GTK_CONTAINER (leftbox), bitratelabel);
    gtk_container_add (GTK_CONTAINER (leftbox), outputlabel);
    
    /* This checks for changes in the entrys */
    g_signal_connect (G_OBJECT (size), "activate", G_CALLBACK ( textchange ), NULL); 
    g_signal_connect (G_OBJECT (length), "activate", G_CALLBACK ( textchange ), NULL); 
    g_signal_connect (G_OBJECT (bitrate), "activate", G_CALLBACK ( textchange ), NULL); 

    /* and the window */
    gtk_widget_show_all (window);

    gtk_widget_grab_focus(size);

    gtk_main ();
    
    return 0;
}
